Polymer({ is : "ti-rov-view",

    behaviors: [rovBehaviors.viewBehaviors],

    properties: {
        id : {
          type: String,
          value: "",
          notify: true
        },
        moduleName: {
          type: String,
          notify: true,
          observer: 'moduleNameChanged'
        },
        shortModuleName: {
          type: String,
          value: ''
        },
        viewData : {
          type: Object,
          value: function() { return {};}
        },
        viewIsShowing: {
          type: Boolean,
          value: false
        },
        viewSelected: {
          type: Boolean,
          value: false
        },
        viewList : {
          type: Object,
          value: function() { return {};}
        },
        viewName: {
          type: String,
          value: ''
        },
        views: {
          type: Array,
          value: function() { return []; }
        },
        hasSelectableColumns : {
          type: Boolean,
          value: false
        },
        dashboardView : {
          type: Boolean,
          value: false
        },
        viewsData : {
          type: Object,
          value: null
        },
        viewRefresh : {
          type: Boolean,
          value: false
        },
        timeoutHandle : {
          type : String,
          value : null
        },
        requestPending : {
          type: Boolean,
          value: false
        },
        pendingViewName : {
          type : String,
          value : ''
        },
        readMemoryOption : {
          type: Boolean,
          value: false
        },
        viewArgs : {
          type : Object,
          value : null
        },
        viewMenuPlaceholder : {
          type : String,
          value: 'Select view'
        },
        readMemoryRefresh : {
            type : Boolean,
            value : false
        },
        readMemoryVisible : {
            type : Boolean,
            value : false
        },
        selectableColumnsVisible : {
            type : Boolean,
            value : false
        },
        refreshVisible : {
            type : Boolean,
            value : false
        }
    },
    ready : function () {
    },
    columnsTapped : function(e) {
        var panel = document.querySelector('ti-rov-panel');
        var clientLeft = panel.isNarrow() ? 0: 256;
        this.$.tiRovTable.columnsContextMenu(e, panel.viewContainer, clientLeft);
    },
    fixedFontChanged : function (e) {
        this.$.tiRovTable.fontChanged(e);
    },
    getModuleName : function () {
        return (this.moduleName);
    },
    getViewCallback : function (error, viewData, module, view) {
        this.requestPending = false;
        if (error == null) {
            if (view.indexOf(':') > 0) {
                view = view.substr(0, view.indexOf(':'));
            }
            if (module == this.moduleName && view == this.viewName) {
                var viewType = this.getViewType(this.moduleName, this.viewName);
                var viewColumns = this.getViewColumns(this.moduleName, this.viewName);
                this.viewIsShowing = this.$.tiRovTable.displayTable(viewData, this.viewName, viewType, viewColumns, this.viewRefresh);
                if (this.viewIsShowing) {
                    this.viewData = viewData;
                    this.hasSelectableColumns = this.$.tiRovTable.hasSelectableColumns();
                }
                else {
                    this.hasSelectableColumns = false;
                }
            }
            else {
                var err = 'Error: module/view mismatch';
                console.error(err);
                this.viewIsShowing = this.$.tiRovTable.displayTable(err, this.viewName, viewType, viewColumns, this.viewRefresh);
            }
        }
        else {
            if (error.indexOf('timeout') == -1 && error.indexOf('cancel') == -1) {
                this.viewIsShowing = this.$.tiRovTable.displayTable(error, this.viewName, viewType, viewColumns, this.viewRefresh);
            }
            else {
                var panel = document.querySelector('ti-rov-panel');
                if (panel.inRepeatRefresh) {
                    panel.stopRepeatRefresh(false);
                }
                if (!this.viewRefresh) {
                    this.viewIsShowing = this.$.tiRovTable.displayTable(error, this.viewName, viewType, viewColumns, this.viewRefresh);
                }
            }
        }
        this.viewSelected = true;
        if (this.$.viewContentDiv.style.position == 'absolute') {
            this.viewToolbarState(this);
        }
        else {
            this.refreshVisible = this.viewSelected;
            this.selectableColumnsVisible = this.hasSelectableColumns;
        }
        document.querySelector('ti-rov-panel').toolbarButtonsState();
        if (this.viewRefresh) {
            this.viewRefresh = false;
        }
    },
    getViewColumns : function(moduleName, viewName) {
        var viewObjs = this.viewList[moduleName];
        var viewColumns = null;
        for (var i = 0; i < viewObjs.length; i++) {
            if (viewObjs[i].name == viewName) {
                viewColumns = viewObjs[i].columns;
                break;
            }
        }
        return (viewColumns);
    },
    getViews : function(moduleName) {
        var views = [];
        var viewObjs = this.viewList[moduleName];
        if (!viewObjs) {
            console.error('No views for module ' + moduleName);
        }
        for (var i = 0; i < viewObjs.length; i++) {
            views.push(viewObjs[i].name);
        }
        return (views);
    },
    getViewsData : function () {
        return(this.$.tiRovTable.getViewsData());
    },
    getViewType : function(moduleName, viewName) {
        var viewObjs = this.viewList[moduleName];
        var viewType = null;
        for (var i = 0; i < viewObjs.length; i++) {
            if (viewObjs[i].name == viewName) {
                viewType = viewObjs[i].type;
                break;
            }
        }
        return (viewType);
    },
    menuSelectView : function (viewName) {
        /* Programmatic way to imitate a physical menu select */
        var index = this.views.indexOf(viewName);
        this.$.viewMenu.selectedItem = this.$.viewListbox.items[index];
        this.$.viewListbox.selected = index;
    },
    moduleNameChanged : function (e) {
        if (!e || e.length == 0) {
            return;
        }
        this.shortModuleName = this.moduleName.substr(this.moduleName.lastIndexOf('.') + 1);
        var rovPanel = document.querySelector('ti-rov-panel');
        this.viewList = rovPanel.getViewList();
        this.views = this.getViews(this.moduleName);

        var viewMenuFont = tiRovStyles.rovViewMenuFontSize + 'pt Open Sans';
        var viewMenuWidth = rovUtils.getStringWidth('Select view', viewMenuFont) + 50;
        this.$.viewMenu.style.width = viewMenuWidth  + 'px';
        this.setMinWidth('Select view', viewMenuFont);
        this.viewIsShowing = false;
        this.readMemoryOption = false;
        this.viewSelected = false;
        this.hasSelectableColumns = false;

        this.readMemoryVisible = false;
        this.selectableColumnsVisible = false;
        this.refreshVisible = false;

        if (this.dashboardView) {
            this.$.tiRovTable.setIsGlobal(false);
            if (this.viewsData != null) {
                this.$.tiRovTable.setViewsData(this.viewsData);
            }
        }
        else {
            this.$.tiRovTable.setIsGlobal(true);
        }
        this.$.tiRovTable.setModuleName(this.moduleName);
    },
    onSave : function() {
        this.saveView(JSON.stringify(this.viewData, null, '\t'));
    },
    onViewSelected : function(e) {
        var viewName = this.$.viewListbox.selectedItem.textContent.trim();
        var refresh = this.viewIsShowing && viewName == this.viewName;
        this.showView(viewName, refresh, e);
    },
    readMemoryDialog : function (e) {
        var addr = this.viewArgs ? this.viewArgs.address : '';
        var len = this.viewArgs ? this.viewArgs.length : '';
        var addrCheck = this.viewArgs ? this.viewArgs.addressCheck : true;
        if (e.currentTarget.id == 'readMemoryButton') {
            this.readMemoryRefresh = this.viewIsShowing;
        }
        this.$.rovViewOptions.readMemoryDialog(e, addr, len, addrCheck, {elem : this, fxn : 'readMemoryDialogCallback'});
    },
    readMemoryDialogCallback : function (address, length, addressCheck) {
        var refresh = this.readMemoryRefresh;
        this.readMemoryRefresh = false;
        if (address == 'cancelled') {
            if (!refresh && this.viewIsShowing) {
                this.showNoTable();
            }
            return;
        }
        if (address.length && length.length) {
            if (!this.viewArgs) {
                this.viewArgs = {};
            }
            this.viewArgs.address = address;
            this.viewArgs.length = length;
            this.viewArgs.addressCheck = addressCheck;
        }
        else {
            this.viewArgs = null;
        }
        this.showView(this.viewName, refresh);
    },
    readMemoryView : function () {
        return (this.shortModuleName == 'Monitor'
                && (this.viewName == 'UChar'
                    || this.viewName == 'Bits16' || this.viewName == 'Bits32'));
    },
    refreshTapped : function() {
        this.showView(this.viewName, true);
    },
    resizerClicked : function (e) {
        rovUtils.initRovResize(e, this.$.viewPaperCard,
                               {elem: this, fxn: this.viewToolbarState},
                               this.$.viewContentDiv);
    },
    saveLastViewSelected : function (viewName) {
        /* Always save last view selected in global storage */
        var viewsData = rovUtils.getLocalStorageObject('viewsData-storage');
        if (!viewsData[this.moduleName]) {
            viewsData[this.moduleName] = {};
        }
        viewsData[this.moduleName].defaultViewName = viewName;
        rovUtils.setLocalStorageObject('viewsData-storage', viewsData);
    },
    setDashboardView : function (isDashboardView) {
        this.dashboardView = isDashboardView;
        this.$.tiRovTable.setIsGlobal(!isDashboardView);
    },
    setMinWidth : function (viewName, viewMenuFont) {
        var viewMenuMaxWidth = rovUtils.getStringWidth(viewName + '000',  viewMenuFont) + 50;
        this.$.viewPaperCard.style.minWidth = (rovUtils.getStringWidth(this.shortModuleName, '14pt Open Sans') +
                                               viewMenuMaxWidth + 20 + 72) + 'px'; /* margin:20, 2 buttons:2x36 */
    },
    setViewList : function (list) {
        this.viewList = list;
    },
    showNoTable : function () {
        this.refreshVisible = false;
        this.selectableColumnsVisible = false;
        this.viewIsShowing = false;
        this.$.tiRovTable.displayTable('  ', this.viewName, null, null, false);
        this.viewSelected = false;
        document.querySelector('ti-rov-panel').toolbarButtonsState();
    },
    showView : function(viewName, refresh, selectedEvent) {
        if (this.requestPending) {
            if (viewName != this.viewName &&
                document.querySelector('ti-rov-panel').inRepeatRefresh) {
                this.pendingViewName = viewName;
            }
            return;
        }
        if (this.pendingViewName != '') {
            if (viewName != this.pendingViewName) {
                viewName = this.pendingViewName;
                refresh = false;
            }
            this.pendingViewName = '';
        }
        this.viewRefresh = refresh;

        /*  Being shown for the first time from a query or a dashboard. Select
         *  the view in the dropdown menu without causing the select event to fire
         */
        if (!refresh && !selectedEvent) {
            var index = this.views.indexOf(viewName);
            this.$.viewMenu._onIronActivate({detail : {item : this.$.viewListbox.items[index]}});
        }
        if (selectedEvent) {
            this.saveLastViewSelected(viewName);
        }
        this.viewName = viewName;
        var viewMenuFont = tiRovStyles.rovViewMenuFontSize + 'pt Open Sans';
        var viewMenuWidth = rovUtils.getStringWidth(viewName, viewMenuFont) + 50;
        this.$.viewMenu.style.width = viewMenuWidth  + 'px';
        this.setMinWidth(viewName, viewMenuFont);
        this.readMemoryOption = this.readMemoryView();
        this.readMemoryVisible = this.readMemoryOption;
        if (this.readMemoryOption) {
            if (this.viewArgs) {
                viewName += ':' + this.viewArgs.address + ',' + this.viewArgs.length +
                            ',' + this.viewArgs.addressCheck;
            }
            else if (selectedEvent) {
                this.readMemoryRefresh = refresh;
                this.readMemoryDialog(selectedEvent);
                return;
            }
        }
        this.requestPending = true;
        this.rovData.getView(this.moduleName, viewName, this);
    },
    viewOptionsMenuClicked : function (e) {
        var dialog = this.$.viewOptionsMenuDialog;
        if (dialog) {

            var left = event.x - 34;
            var width = 235; /* ballpark from styling in html */
            if ((left + width) > document.documentElement.clientWidth) {
                left -= width;
            }
            dialog.style.left = left + 'px';
            var height = 30 + 33 + 10; /* onSave button, fixed font checkbox, padding */
            if (this.$.readMemoryDialog.hidden == false) {
                height += 30;
            }
            if (this.$.columnsTapped.hidden == false) {
                height += 30;
            }
            if (this.$.refreshTapped.hidden == false) {
                height += 30;
            }
            var top = e.y - this.$.closeButton.offsetHeight;
            if (top + height > document.querySelector('ti-rov-panel').viewContainer.clientHeight) {
              top = top - height;
              if (top < 0) {
                  top = 0;
              }
            }
            dialog.style.top = top + 'px';
            dialog.open();
            var viewsData = this.getViewsData();
            this.$.fixedFontCheckbox.checked = viewsData[this.moduleName + '.' + this.viewName].fixedFont;
            dialog.style.zIndex = String(document.querySelector('ti-rov-panel').getMaxZindex() + 1);
        }
    },
    viewOptionsMenuItemSelected : function (e) {
        var selected = e.currentTarget.selectedItem.id;
        this.$.viewOptionsMenu.selected = -1;
        this.$.viewOptionsMenuDialog.close();
        if (selected != 'fixedFontItem') {
            this[selected](e); /* id of the item is the function name */
        }
    },
    viewToolbarState : function () {
        var toolbarLeft = this.$.viewMenu.offsetLeft + this.$.viewMenu.offsetWidth + 8;
        var viewWidth = this.offsetWidth;
        var numButtons = 1; /* close button */
        var stateArr = [];
        if (this.hasSelectableColumns) {  /* columns button */
            ++numButtons;
            stateArr.push({btnVisible : 'selectableColumnsVisible', item : 'columnsTapped'});
        }
        if (this.readMemoryOption) {  /* read memory button */
            ++numButtons;
            stateArr.push({btnVisible : 'readMemoryVisible', item : 'readMemoryDialog'});
        }
        if (this.viewSelected) {   /* refresh button */
            ++numButtons;
            stateArr.push({btnVisible : 'refreshVisible', item : 'refreshTapped'});
        }
        if (this.viewIsShowing) {
            ++numButtons; /* for viewOptionsMenu */
        }
        var buttonsVisible = Math.round(((viewWidth - toolbarLeft) / 36) - .9);
        if (buttonsVisible > numButtons) {
            buttonsVisible = numButtons;
        }
        var diffCount = (buttonsVisible < numButtons) ? numButtons - buttonsVisible : 0;
        for (var i = 0; i < stateArr.length; i++) {
            if (i < diffCount) {
                this[stateArr[i].btnVisible] = false;
                this.$[stateArr[i].item].hidden = false;
            }
            else {
                this[stateArr[i].btnVisible] = true;
                this.$[stateArr[i].item].hidden = true;
            }
        }
    }
});
